@@ -68,10 +68,10 @@ class tool_behat_renderer extends plugin_renderer_base {
         $html .= $this->output->box_start();
         $html .= html_writer::tag('h1', get_string('infoheading', 'tool_behat'));
         $html .= html_writer::tag('div', get_string('aim', 'tool_behat'));
-        $html .= html_writer::empty_tag('div');
-        $html .= html_writer::empty_tag('ul');
-        $html .= html_writer::empty_tag('li');
-        $html .= implode(html_writer::end_tag('li') . html_writer::empty_tag('li'), $infos);
+        $html .= html_writer::start_tag('div');
+        $html .= html_writer::start_tag('ul');
+        $html .= html_writer::start_tag('li');
+        $html .= implode(html_writer::end_tag('li') . html_writer::start_tag('li'), $infos);
         $html .= html_writer::end_tag('li');
         $html .= html_writer::end_tag('ul');
         $html .= html_writer::end_tag('div');
@@ -105,6 +105,15 @@ class tool_behat_renderer extends plugin_renderer_base {
                 $stepsdefinitions
             );
 
+            // Replace simple OR options.
+            $regex = '#\(\?P<[^>]+>([^\)|]+\|[^\)]+)\)#';
+            $stepsdefinitions = preg_replace_callback($regex,
+                function($matches){
+                    return html_writer::select(explode('|', $matches[1]), uniqid());
+                },
+                $stepsdefinitions
+            );
+
         }
 
         // Steps definitions.
