@@ -39,5 +39,18 @@ function xmldb_auth_cas_upgrade($oldversion) {
         upgrade_plugin_savepoint(true, 2013052100, 'auth', 'cas');
     }
 
+    if ($oldversion < 2013091700) {
+        // The value of the phpCAS language constants has changed from
+        // 'langname' to 'CAS_Languages_Langname'.
+        if ($cas_language = get_config('auth/cas', 'language')) {
+            set_config('language', 'CAS_Languages_'.ucfirst($cas_language), 'auth/cas');
+        }
+
+        upgrade_plugin_savepoint(true, 2013091700, 'auth', 'cas');
+    }
+
+    // Moodle v2.6.0 release upgrade line.
+    // Put any upgrade step following this.
+
     return true;
 }
