@@ -223,7 +223,7 @@
 
         if(empty($title)){
             // no title present, use portion of description
-            $title = textlib::substr(strip_tags($description), 0, 20) . '...';
+            $title = core_text::substr(strip_tags($description), 0, 20) . '...';
         }else{
             $title = break_up_long_words($title, 30);
         }
@@ -249,13 +249,13 @@
 
             if($this->config->display_description && !empty($description)){
 
-                $description = break_up_long_words($description, 30);
-
                 $formatoptions = new stdClass();
                 $formatoptions->para = false;
 
                 $r.= html_writer::start_tag('div',array('class'=>'description'));
-                    $r.= format_text($description, FORMAT_HTML, $formatoptions, $this->page->course->id);
+                    $description = format_text($description, FORMAT_HTML, $formatoptions, $this->page->course->id);
+                    $description = break_up_long_words($description, 30);
+                    $r.= $description;
                 $r.= html_writer::end_tag('div');
             }
         $r.= html_writer::end_tag('li');
@@ -272,10 +272,10 @@
      */
     function format_title($title,$max=64) {
 
-        if (textlib::strlen($title) <= $max) {
+        if (core_text::strlen($title) <= $max) {
             return s($title);
         } else {
-            return s(textlib::substr($title,0,$max-3).'...');
+            return s(core_text::substr($title,0,$max-3).'...');
         }
     }
 
