@@ -109,15 +109,6 @@ class oci_native_moodle_database extends moodle_database {
         return get_string('nativeocihelp', 'install');
     }
 
-    /**
-     * Returns localised database description
-     * Note: can be used before connect()
-     * @return string
-     */
-    public function get_configuration_hints() {
-        return get_string('databasesettingssub_oci', 'install');
-    }
-
     /**
      * Diagnose database and tables, this function is used
      * to verify database and driver settings, db engine types, etc.
@@ -415,7 +406,7 @@ class oci_native_moodle_database extends moodle_database {
         oci_free_statement($stmt);
         $records = array_map('strtolower', $records['TABLE_NAME']);
         foreach ($records as $tablename) {
-            if ($this->prefix !== '') {
+            if ($this->prefix !== false && $this->prefix !== '') {
                 if (strpos($tablename, $this->prefix) !== 0) {
                     continue;
                 }
@@ -574,7 +565,7 @@ class oci_native_moodle_database extends moodle_database {
                         $info->meta_type     = 'I';
                         $info->unique        = null;
                     }
-                    $info->scale = null;
+                    $info->scale = 0;
 
                 } else {
                     //float
@@ -673,7 +664,7 @@ class oci_native_moodle_database extends moodle_database {
         }
 
         if ($usecache) {
-            $result = $cache->set($table, $structure);
+            $cache->set($table, $structure);
         }
 
         return $structure;
@@ -1731,6 +1722,10 @@ class oci_native_moodle_database extends moodle_database {
         }
     }
 
+    public function session_lock_supported() {
+        return true;
+    }
+
     /**
      * Obtain session lock
      * @param int $rowid id of the row with session record
