@@ -34,6 +34,11 @@ require_once(__DIR__.'/../../testing/classes/util.php');
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class phpunit_util extends testing_util {
+    /**
+     * @var int last value of db writes counter, used for db resetting
+     */
+    public static $lastdbwrites = null;
+
     /** @var array An array of original globals, restored after each test */
     protected static $globals = array();
 
@@ -46,6 +51,9 @@ class phpunit_util extends testing_util {
     /** @var phpunit_phpmailer_sink alternative target for phpmailer messaging */
     protected static $phpmailersink = null;
 
+    /** @var phpunit_message_sink alternative target for moodle messaging */
+    protected static $eventsink = null;
+
     /**
      * @var array Files to skip when resetting dataroot folder
      */
@@ -72,7 +80,7 @@ class phpunit_util extends testing_util {
             initialise_cfg();
             return;
         }
-        if ($dbhash !== self::get_version_hash()) {
+        if ($dbhash !== core_component::get_all_versions_hash()) {
             // do not set CFG - the only way forward is to drop and reinstall
             return;
         }
@@ -101,6 +109,9 @@ class phpunit_util extends testing_util {
         // Stop any message redirection.
         phpunit_util::stop_phpmailer_redirection();
 
+        // Stop any message redirection.
+        phpunit_util::stop_event_redirection();
+
         // Release memory and indirectly call destroy() methods to release resource handles, etc.
         gc_collect_cycles();
 
@@ -185,18 +196,16 @@ class phpunit_util extends testing_util {
         $user = new stdClass();
         $user->id = 0;
         $user->mnethostid = $CFG->mnet_localhost_id;
-        session_set_user($user);
+        \core\session\manager::set_user($user);
 
         // reset all static caches
+        \core\event\manager::phpunit_reset();
         accesslib_clear_all_caches(true);
         get_string_manager()->reset_caches(true);
         reset_text_filters_cache(true);
         events_get_handlers('reset');
-        textlib::reset_caches();
+        core_text::reset_caches();
         get_message_processors(false, true);
-        if (class_exists('repository')) {
-            repository::reset_caches();
-        }
         filter_manager::reset_caches();
         //TODO MDL-25290: add more resets here and probably refactor them to new core function
 
@@ -208,14 +217,14 @@ class phpunit_util extends testing_util {
         get_fast_modinfo(0, 0, true);
 
         // Reset other singletons.
-        if (class_exists('plugin_manager')) {
-            plugin_manager::reset_caches(true);
+        if (class_exists('core_plugin_manager')) {
+            core_plugin_manager::reset_caches(true);
         }
-        if (class_exists('available_update_checker')) {
-            available_update_checker::reset_caches(true);
+        if (class_exists('\core\update\checker')) {
+            \core\update\checker::reset_caches(true);
         }
-        if (class_exists('available_update_deployer')) {
-            available_update_deployer::reset_caches(true);
+        if (class_exists('\core\update\deployer')) {
+            \core\update\deployer::reset_caches(true);
         }
 
         // purge dataroot directory
@@ -242,6 +251,27 @@ class phpunit_util extends testing_util {
         }
     }
 
+    /**
+     * Reset all database tables to default values.
+     * @static
+     * @return bool true if reset done, false if skipped
+     */
+    public static function reset_database() {
+        global $DB;
+
+        if (!is_null(self::$lastdbwrites) and self::$lastdbwrites == $DB->perf_get_writes()) {
+            return false;
+        }
+
+        if (!parent::reset_database()) {
+            return false;
+        }
+
+        self::$lastdbwrites = $DB->perf_get_writes();
+
+        return true;
+    }
+
     /**
      * Called during bootstrap only!
      * @internal
@@ -380,6 +410,10 @@ class phpunit_util extends testing_util {
 
         install_cli_database($options, false);
 
+        // We need to keep the installed dataroot filedir files.
+        // So each time we reset the dataroot before running a test, the default files are still installed.
+        self::save_original_data_files();
+
         // install timezone info
         $timezones = get_records_csv($CFG->libdir.'/timezone.txt', 'timezone');
         update_timezone_records($timezones);
@@ -407,10 +441,10 @@ class phpunit_util extends testing_util {
 
         $suites = '';
 
-        $plugintypes = get_plugin_types();
+        $plugintypes = core_component::get_plugin_types();
         ksort($plugintypes);
         foreach ($plugintypes as $type=>$unused) {
-            $plugs = get_plugin_list($type);
+            $plugs = core_component::get_plugin_list($type);
             ksort($plugs);
             foreach ($plugs as $plug=>$fullplug) {
                 if (!file_exists("$fullplug/tests/")) {
@@ -538,6 +572,7 @@ class phpunit_util extends testing_util {
      */
     public static function reset_debugging() {
         self::$debuggings = array();
+        set_debugging(DEBUG_DEVELOPER);
     }
 
     /**
@@ -660,4 +695,57 @@ class phpunit_util extends testing_util {
             self::$phpmailersink->add_message($message);
         }
     }
+
+    /**
+     * Start event redirection.
+     *
+     * @private
+     * Note: Do not call directly from tests,
+     *       use $sink = $this->redirectEvents() instead.
+     *
+     * @return phpunit_event_sink
+     */
+    public static function start_event_redirection() {
+        if (self::$eventsink) {
+            self::stop_event_redirection();
+        }
+        self::$eventsink = new phpunit_event_sink();
+        return self::$eventsink;
+    }
+
+    /**
+     * End event redirection.
+     *
+     * @private
+     * Note: Do not call directly from tests,
+     *       use $sink->close() instead.
+     */
+    public static function stop_event_redirection() {
+        self::$eventsink = null;
+    }
+
+    /**
+     * Are events redirected to some sink?
+     *
+     * Note: to be called from \core\event\base only!
+     *
+     * @private
+     * @return bool
+     */
+    public static function is_redirecting_events() {
+        return !empty(self::$eventsink);
+    }
+
+    /**
+     * To be called from \core\event\base only!
+     *
+     * @private
+     * @param \core\event\base $event record from event_read table
+     * @return bool true means send event, false means event "sent" to sink.
+     */
+    public static function event_triggered(\core\event\base $event) {
+        if (self::$eventsink) {
+            self::$eventsink->add_event($event);
+        }
+    }
 }
