@@ -815,8 +815,16 @@ class t3lib_cs {
 	 * @return	string		Output string
 	 */
 	function entities_to_utf8($str, $alsoStdHtmlEnt = FALSE) {
+		// Workaround for #39287: 3rd parameter for get_html_translation_table() was only added in PHP 5.3.4 and later
+		// see http://php.net/manual/en/function.get-html-translation-table.php
+		$applyPhpCompatibilityFix = version_compare(phpversion(), '5.3.4', '<');
+
 		if ($alsoStdHtmlEnt) {
-			$trans_tbl = array_flip(get_html_translation_table(HTML_ENTITIES, ENT_COMPAT, 'UTF-8'));
+			if ($applyPhpCompatibilityFix === TRUE) {
+				$trans_tbl = array_flip(get_html_translation_table(HTML_ENTITIES, ENT_COMPAT));
+			} else {
+				$trans_tbl = array_flip(get_html_translation_table(HTML_ENTITIES, ENT_COMPAT, 'UTF-8'));
+			}
 		}
 
 		$token = md5(microtime());
@@ -837,7 +845,11 @@ class t3lib_cs {
 				}
 				$parts[$k] = $this->UnumberToChar($v);
 			} elseif ($alsoStdHtmlEnt && isset($trans_tbl['&' . $v . ';'])) { // Other entities:
-				$parts[$k] = $trans_tbl['&' . $v . ';'];
+				$v = $trans_tbl['&' . $v . ';'];
+				if ($applyPhpCompatibilityFix === TRUE) {
+					$v = $this->utf8_encode($v, 'iso-8859-1');
+				}
+				$parts[$k] = $v;
 			} else { // No conversion:
 				$parts[$k] = '&' . $v . ';';
 			}
