@@ -94,6 +94,7 @@ class upload_manager {
             $course = $SITE;
         }
 
+        $this->config = new stdClass();
         $this->config->deleteothers = $deleteothers;
         $this->config->handlecollisions = $handlecollisions;
         $this->config->recoverifmultiple = $recoverifmultiple;
@@ -582,6 +583,8 @@ function clam_handle_infected_file($file, $userid=0, $basiconly=false) {
  * @return boolean
  */
 function clam_replace_infected_file($file) {
+    global $CFG;
+
     $newcontents = get_string('virusplaceholder');
     if (!$f = fopen($file, 'w')) {
         return false;
@@ -589,6 +592,7 @@ function clam_replace_infected_file($file) {
     if (!fwrite($f, $newcontents)) {
         return false;
     }
+    @chmod($file, $CFG->filepermissions);
     return true;
 }
 
