@@ -15,7 +15,7 @@
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * User logout event handler definition.
+ * Event observers definition.
  *
  * @package mod_chat
  * @category event
@@ -23,12 +23,12 @@
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-/* List of handlers */
-$handlers = array (
-    'user_logout' => array (
-        'handlerfile'      => '/mod/chat/lib.php',
-        'handlerfunction'  => 'chat_user_logout',
-        'schedule'         => 'instant',
-        'internal'         => 1,
-    ),
+$observers = array(
+
+    // User logging out.
+    array(
+        'eventname' => '\core\event\user_loggedout',
+        'callback' => 'chat_user_logout',
+        'includefile' => '/mod/chat/lib.php'
+    )
 );
