@@ -81,8 +81,8 @@ $context = context_module::instance($cm->id);
 if (empty($cm->visible) and !has_capability('moodle/course:viewhiddenactivities', $context)) {
     $strdatabases = get_string("modulenameplural", "data");
 
-    $PAGE->set_title(format_string($data->name));
-    $PAGE->set_heading(format_string($course->fullname));
+    $PAGE->set_title($data->name);
+    $PAGE->set_heading($course->fullname);
     echo $OUTPUT->header();
     notice(get_string("activityiscurrentlyhidden"));
 }
@@ -249,8 +249,9 @@ if ($datarecord = data_submitted() and confirm_sesskey()) {
 /// Print the page header
 
 echo $OUTPUT->header();
+echo $OUTPUT->heading(format_string($data->name), 2);
+echo $OUTPUT->box(format_module_intro('data', $data, $cm->id), 'generalbox', 'intro');
 groups_print_activity_menu($cm, $CFG->wwwroot.'/mod/data/edit.php?d='.$data->id);
-echo $OUTPUT->heading(format_string($data->name));
 
 /// Print the tabs
 
@@ -275,7 +276,7 @@ echo '<input name="sesskey" value="'.sesskey().'" type="hidden" />';
 echo $OUTPUT->box_start('generalbox boxaligncenter boxwidthwide');
 
 if (!$rid){
-    echo $OUTPUT->heading(get_string('newentry','data'), 2);
+    echo $OUTPUT->heading(get_string('newentry','data'), 3);
 }
 
 /******************************************
