@@ -57,6 +57,10 @@ class mod_lti_mod_form extends moodleform_mod {
     public function definition() {
         global $DB, $PAGE, $OUTPUT, $USER, $COURSE;
 
+        if ($type = optional_param('type', false, PARAM_ALPHA)) {
+            component_callback("ltisource_$type", 'add_instance_hook');
+        }
+
         $this->typeid = 0;
 
         $mform =& $this->_form;
@@ -83,6 +87,7 @@ class mod_lti_mod_form extends moodleform_mod {
 
         $mform->addElement('checkbox', 'showtitlelaunch', '&nbsp;', ' ' . get_string('display_name', 'lti'));
         $mform->setAdvanced('showtitlelaunch');
+        $mform->setDefault('showtitlelaunch', true);
         $mform->addHelpButton('showtitlelaunch', 'display_name', 'lti');
 
         $mform->addElement('checkbox', 'showdescriptionlaunch', '&nbsp;', ' ' . get_string('display_description', 'lti'));
@@ -114,6 +119,9 @@ class mod_lti_mod_form extends moodleform_mod {
         $mform->setAdvanced('securetoolurl');
         $mform->addHelpButton('securetoolurl', 'secure_launch_url', 'lti');
 
+        $mform->addElement('hidden', 'urlmatchedtypeid', '', array( 'id' => 'id_urlmatchedtypeid' ));
+        $mform->setType('urlmatchedtypeid', PARAM_INT);
+
         $launchoptions=array();
         $launchoptions[LTI_LAUNCH_CONTAINER_DEFAULT] = get_string('default', 'lti');
         $launchoptions[LTI_LAUNCH_CONTAINER_EMBED] = get_string('embed', 'lti');
