@@ -90,7 +90,7 @@ class quiz_access_manager {
      * @return array of all the installed rule class names.
      */
     protected static function get_rule_classes() {
-        return get_plugin_list_with_class('quizaccess', '', 'rule.php');
+        return core_component::get_plugin_list_with_class('quizaccess', '', 'rule.php');
     }
 
     /**
@@ -132,10 +132,10 @@ class quiz_access_manager {
      * @return array $errors the updated $errors array.
      */
     public static function validate_settings_form_fields(array $errors,
-            array $data, $files, mod_quiz_mod_form $this) {
+            array $data, $files, mod_quiz_mod_form $quizform) {
 
         foreach (self::get_rule_classes() as $rule) {
-            $errors = $rule::validate_settings_form_fields($errors, $data, $files, $this);
+            $errors = $rule::validate_settings_form_fields($errors, $data, $files, $quizform);
         }
 
         return $errors;
