@@ -310,7 +310,7 @@ class quiz_overview_report extends quiz_attempts_report {
      */
     protected function finish_regrade($nexturl) {
         global $OUTPUT, $PAGE;
-        echo $OUTPUT->heading(get_string('regradecomplete', 'quiz_overview'));
+        echo $OUTPUT->heading(get_string('regradecomplete', 'quiz_overview'), 3);
         echo $OUTPUT->continue_button($nexturl);
         echo $OUTPUT->footer();
         die();
@@ -320,7 +320,7 @@ class quiz_overview_report extends quiz_attempts_report {
      * Unlock the session and allow the regrading process to run in the background.
      */
     protected function unlock_session() {
-        session_get_instance()->write_close();
+        \core\session\manager::write_close();
         ignore_user_abort(true);
     }
 
