@@ -53,12 +53,12 @@ class mod_wiki_renderer extends plugin_renderer_base {
     }
 
     public function search_result($records, $subwiki) {
-        global $CFG, $PAGE;
+        global $CFG;
         $table = new html_table();
-        $context = context_module::instance($PAGE->cm->id);
+        $context = context_module::instance($this->page->cm->id);
         $strsearchresults = get_string('searchresult', 'wiki');
         $totalcount = count($records);
-        $html = $this->output->heading("$strsearchresults $totalcount");
+        $html = $this->output->heading("$strsearchresults $totalcount", 3);
         foreach ($records as $page) {
             $table->head = array('title' => format_string($page->title) . ' (' . html_writer::link($CFG->wwwroot . '/mod/wiki/view.php?pageid=' . $page->id, get_string('view', 'wiki')) . ')');
             $table->align = array('title' => 'left');
@@ -294,7 +294,7 @@ class mod_wiki_renderer extends plugin_renderer_base {
         $html = '';
         $link = new moodle_url('/mod/wiki/prettyview.php', array('pageid' => $page->id));
         $html .= $this->output->container_start('wiki_right');
-        $html .= $this->output->action_link($link, get_string('prettyprint', 'wiki'), new popup_action('click', $link));
+        $html .= $this->output->action_link($link, get_string('prettyprint', 'wiki'), new popup_action('click', $link), array('class' => 'printicon'));
         $html .= $this->output->container_end();
         return $html;
     }
@@ -302,17 +302,27 @@ class mod_wiki_renderer extends plugin_renderer_base {
     public function wiki_print_subwiki_selector($wiki, $subwiki, $page, $pagetype = 'view') {
         global $CFG, $USER;
         require_once($CFG->dirroot . '/user/lib.php');
+        $cm = get_coursemodule_from_instance('wiki', $wiki->id);
+
         switch ($pagetype) {
         case 'files':
-            $baseurl = new moodle_url('/mod/wiki/files.php');
+            $baseurl = new moodle_url('/mod/wiki/files.php',
+                    array('wid' => $wiki->id, 'title' => $page->title, 'pageid' => $page->id));
+            break;
+        case 'search':
+            $search = optional_param('searchstring', null, PARAM_ALPHANUMEXT);
+            $searchcontent = optional_param('searchwikicontent', 0, PARAM_INT);
+            $baseurl = new moodle_url('/mod/wiki/search.php',
+                    array('cmid' => $cm->id, 'courseid' => $cm->course,
+                        'searchstring' => $search, 'searchwikicontent' => $searchcontent));
             break;
         case 'view':
         default:
-            $baseurl = new moodle_url('/mod/wiki/view.php');
+            $baseurl = new moodle_url('/mod/wiki/view.php',
+                    array('wid' => $wiki->id, 'title' => $page->title));
             break;
         }
 
-        $cm = get_coursemodule_from_instance('wiki', $wiki->id);
         $context = context_module::instance($cm->id);
         // @TODO: A plenty of duplicated code below this lines.
         // Create private functions.
@@ -337,14 +347,9 @@ class mod_wiki_renderer extends plugin_renderer_base {
                     }
 
                     echo $this->output->container_start('wiki_right');
-                    $params = array('wid' => $wiki->id, 'title' => $page->title);
-                    if ($pagetype == 'files') {
-                        $params['pageid'] = $page->id;
-                    }
-                    $baseurl->params($params);
                     $name = 'uid';
                     $selected = $subwiki->userid;
-                    echo $this->output->single_select($baseurl, $name, $options, $selected);
+                    echo $this->output->single_select($baseurl, $name, $options, $selected, null);
                     echo $this->output->container_end();
                 }
                 return;
@@ -356,12 +361,6 @@ class mod_wiki_renderer extends plugin_renderer_base {
             if ($wiki->wikimode == 'collaborative') {
                 // We need to print a select to choose a course group
 
-                $params = array('wid'=>$wiki->id, 'title'=>$page->title);
-                if ($pagetype == 'files') {
-                    $params['pageid'] = $page->id;
-                }
-                $baseurl->params($params);
-
                 echo $this->output->container_start('wiki_right');
                 groups_print_activity_menu($cm, $baseurl);
                 echo $this->output->container_end();
@@ -397,14 +396,9 @@ class mod_wiki_renderer extends plugin_renderer_base {
                     }
                 }
                 echo $this->output->container_start('wiki_right');
-                $params = array('wid' => $wiki->id, 'title' => $page->title);
-                if ($pagetype == 'files') {
-                    $params['pageid'] = $page->id;
-                }
-                $baseurl->params($params);
                 $name = 'groupanduser';
                 $selected = $subwiki->groupid . '-' . $subwiki->userid;
-                echo $this->output->single_select($baseurl, $name, $options, $selected);
+                echo $this->output->single_select($baseurl, $name, $options, $selected, null);
                 echo $this->output->container_end();
 
                 return;
@@ -417,11 +411,6 @@ class mod_wiki_renderer extends plugin_renderer_base {
             if ($wiki->wikimode == 'collaborative') {
                 // We need to print a select to choose a course group
                 // moodle_url will take care of encoding for us
-                $params = array('wid'=>$wiki->id, 'title'=>$page->title);
-                if ($pagetype == 'files') {
-                    $params['pageid'] = $page->id;
-                }
-                $baseurl->params($params);
 
                 echo $this->output->container_start('wiki_right');
                 groups_print_activity_menu($cm, $baseurl);
@@ -444,14 +433,9 @@ class mod_wiki_renderer extends plugin_renderer_base {
                 }
 
                 echo $this->output->container_start('wiki_right');
-                $params = array('wid' => $wiki->id, 'title' => $page->title);
-                if ($pagetype == 'files') {
-                    $params['pageid'] = $page->id;
-                }
-                $baseurl->params($params);
                 $name = 'groupanduser';
                 $selected = $subwiki->groupid . '-' . $subwiki->userid;
-                echo $this->output->single_select($baseurl, $name, $options, $selected);
+                echo $this->output->single_select($baseurl, $name, $options, $selected, null);
                 echo $this->output->container_end();
 
                 return;
@@ -469,6 +453,10 @@ class mod_wiki_renderer extends plugin_renderer_base {
     }
 
     function menu_map($pageid, $currentselect) {
+        if (empty($currentselect)) {
+            // The wiki uses digit number to match the options and 5 is the default one.
+            $currentselect = 5;
+        }
         $options = array('contributions', 'links', 'orphaned', 'pageindex', 'pagelist', 'updatedpages');
         $items = array();
         foreach ($options as $opt) {
@@ -478,7 +466,7 @@ class mod_wiki_renderer extends plugin_renderer_base {
         foreach ($items as $key => $item) {
             $selectoptions[$key + 1] = $item;
         }
-        $select = new single_select(new moodle_url('/mod/wiki/map.php', array('pageid' => $pageid)), 'option', $selectoptions, $currentselect);
+        $select = new single_select(new moodle_url('/mod/wiki/map.php', array('pageid' => $pageid)), 'option', $selectoptions, $currentselect, null);
         $select->label = get_string('mapmenu', 'wiki') . ': ';
         return $this->output->container($this->output->render($select), 'midpad');
     }
@@ -509,7 +497,7 @@ class mod_wiki_renderer extends plugin_renderer_base {
         foreach ($items as $key => $item) {
             $selectoptions[$key + 1] = $item;
         }
-        $select = new single_select(new moodle_url('/mod/wiki/admin.php', array('pageid' => $pageid)), 'option', $selectoptions, $currentselect);
+        $select = new single_select(new moodle_url('/mod/wiki/admin.php', array('pageid' => $pageid)), 'option', $selectoptions, $currentselect, null);
         $select->label = get_string('adminmenu', 'wiki') . ': ';
         return $this->output->container($this->output->render($select), 'midpad');
     }
