@@ -106,11 +106,12 @@ class question_engine_data_mapper {
         $record->variant = $qa->get_variant();
         $record->maxmark = $qa->get_max_mark();
         $record->minfraction = $qa->get_min_fraction();
+        $record->maxfraction = $qa->get_max_fraction();
         $record->flagged = $qa->is_flagged();
         $record->questionsummary = $qa->get_question_summary();
-        if (textlib::strlen($record->questionsummary) > question_bank::MAX_SUMMARY_LENGTH) {
+        if (core_text::strlen($record->questionsummary) > question_bank::MAX_SUMMARY_LENGTH) {
             // It seems some people write very long quesions! MDL-30760
-            $record->questionsummary = textlib::substr($record->questionsummary,
+            $record->questionsummary = core_text::substr($record->questionsummary,
                     0, question_bank::MAX_SUMMARY_LENGTH - 3) . '...';
         }
         $record->rightanswer = $qa->get_right_answer_summary();
@@ -259,6 +260,7 @@ SELECT
     qa.variant,
     qa.maxmark,
     qa.minfraction,
+    qa.maxfraction,
     qa.flagged,
     qa.questionsummary,
     qa.rightanswer,
@@ -318,6 +320,7 @@ SELECT
     qa.variant,
     qa.maxmark,
     qa.minfraction,
+    qa.maxfraction,
     qa.flagged,
     qa.questionsummary,
     qa.rightanswer,
@@ -359,16 +362,16 @@ ORDER BY
      * Load information about the latest state of each question from the database.
      *
      * @param qubaid_condition $qubaids used to restrict which usages are included
-     * in the query. See {@link qubaid_condition}.
-     * @param array $slots A list of slots for the questions you want to konw about.
+     *                                  in the query. See {@link qubaid_condition}.
+     * @param array            $slots   A list of slots for the questions you want to know about.
+     * @param string|null      $fields
      * @return array of records. See the SQL in this function to see the fields available.
      */
-    public function load_questions_usages_latest_steps(qubaid_condition $qubaids, $slots) {
+    public function load_questions_usages_latest_steps(qubaid_condition $qubaids, $slots, $fields = null) {
         list($slottest, $params) = $this->db->get_in_or_equal($slots, SQL_PARAMS_NAMED, 'slot');
 
-        $records = $this->db->get_records_sql("
-SELECT
-    qas.id,
+        if ($fields === null) {
+            $fields =  "qas.id,
     qa.id AS questionattemptid,
     qa.questionusageid,
     qa.slot,
@@ -377,6 +380,7 @@ SELECT
     qa.variant,
     qa.maxmark,
     qa.minfraction,
+    qa.maxfraction,
     qa.flagged,
     qa.questionsummary,
     qa.rightanswer,
@@ -387,11 +391,17 @@ SELECT
     qas.state,
     qas.fraction,
     qas.timecreated,
-    qas.userid
+    qas.userid";
+
+        }
+
+        $records = $this->db->get_records_sql("
+SELECT
+    {$fields}
 
 FROM {$qubaids->from_question_attempts('qa')}
-JOIN {question_attempt_steps} qas ON
-        qas.id = {$this->latest_step_for_qa_subquery()}
+JOIN {question_attempt_steps} qas ON qas.questionattemptid = qa.id
+        AND qas.sequencenumber = {$this->latest_step_for_qa_subquery()}
 
 WHERE
     {$qubaids->where()} AND
@@ -428,8 +438,8 @@ SELECT
     COUNT(1) AS numattempts
 
 FROM {$qubaids->from_question_attempts('qa')}
-JOIN {question_attempt_steps} qas ON
-        qas.id = {$this->latest_step_for_qa_subquery()}
+JOIN {question_attempt_steps} qas ON qas.questionattemptid = qa.id
+        AND qas.sequencenumber = {$this->latest_step_for_qa_subquery()}
 JOIN {question} q ON q.id = qa.questionid
 
 WHERE
@@ -500,7 +510,7 @@ ORDER BY
      */
     public function load_questions_usages_where_question_in_state(
             qubaid_condition $qubaids, $summarystate, $slot, $questionid = null,
-            $orderby = 'random', $params, $limitfrom = 0, $limitnum = null) {
+            $orderby = 'random', $params = array(), $limitfrom = 0, $limitnum = null) {
 
         $extrawhere = '';
         if ($questionid) {
@@ -521,26 +531,28 @@ ORDER BY
             $sqlorderby = '';
         }
 
-        // We always want the total count, as well as the partcular list of ids,
-        // based on the paging and sort order. Becuase the list of ids is never
-        // going to be too rediculously long. My worst-case scenario is
-        // 10,000 students in the coures, each doing 5 quiz attempts. That
+        // We always want the total count, as well as the partcular list of ids
+        // based on the paging and sort order. Because the list of ids is never
+        // going to be too ridiculously long. My worst-case scenario is
+        // 10,000 students in the course, each doing 5 quiz attempts. That
         // is a 50,000 element int => int array, which PHP seems to use 5MB
-        // memeory to store on a 64 bit server.
+        // memory to store on a 64 bit server.
+        $qubaidswhere = $qubaids->where(); // Must call this before params.
         $params += $qubaids->from_where_params();
         $params['slot'] = $slot;
+
         $qubaids = $this->db->get_records_sql_menu("
 SELECT
     qa.questionusageid,
     1
 
 FROM {$qubaids->from_question_attempts('qa')}
-JOIN {question_attempt_steps} qas ON
-        qas.id = {$this->latest_step_for_qa_subquery()}
+JOIN {question_attempt_steps} qas ON qas.questionattemptid = qa.id
+        AND qas.sequencenumber = {$this->latest_step_for_qa_subquery()}
 JOIN {question} q ON q.id = qa.questionid
 
 WHERE
-    {$qubaids->where()} AND
+    {$qubaidswhere} AND
     qa.slot = :slot
     $extrawhere
 
@@ -577,7 +589,7 @@ $sqlorderby
             $slotwhere = " AND qa.slot $slottest";
         } else {
             $slotwhere = '';
-            $params = array();
+            $slotsparams = array();
         }
 
         list($statetest, $stateparams) = $this->db->get_in_or_equal(array(
@@ -597,8 +609,8 @@ SELECT
     COUNT(1) AS numaveraged
 
 FROM {$qubaids->from_question_attempts('qa')}
-JOIN {question_attempt_steps} qas ON
-        qas.id = {$this->latest_step_for_qa_subquery()}
+JOIN {question_attempt_steps} qas ON qas.questionattemptid = qa.id
+        AND qas.sequencenumber = {$this->latest_step_for_qa_subquery()}
 
 WHERE
     {$qubaids->where()}
@@ -635,6 +647,7 @@ SELECT
     qa.variant,
     qa.maxmark,
     qa.minfraction,
+    qa.maxfraction,
     qa.flagged,
     qa.questionsummary,
     qa.rightanswer,
@@ -702,6 +715,7 @@ ORDER BY
         $record->id = $qa->get_database_id();
         $record->maxmark = $qa->get_max_mark();
         $record->minfraction = $qa->get_min_fraction();
+        $record->maxfraction = $qa->get_max_fraction();
         $record->flagged = $qa->is_flagged();
         $record->questionsummary = $qa->get_question_summary();
         $record->rightanswer = $qa->get_right_answer_summary();
@@ -912,10 +926,11 @@ ORDER BY
         // NULL total into a 0.
         return "SELECT COALESCE(SUM(qa.maxmark * qas.fraction), 0)
             FROM {question_attempts} qa
-            JOIN {question_attempt_steps} qas ON qas.id = (
-                SELECT MAX(summarks_qas.id)
-                  FROM {question_attempt_steps} summarks_qas
-                 WHERE summarks_qas.questionattemptid = qa.id
+            JOIN {question_attempt_steps} qas ON qas.questionattemptid = qa.id
+                    AND qas.sequencenumber = (
+                            SELECT MAX(summarks_qas.sequencenumber)
+                              FROM {question_attempt_steps} summarks_qas
+                             WHERE summarks_qas.questionattemptid = qa.id
             )
             WHERE qa.questionusageid = $qubaid
             HAVING COUNT(CASE
@@ -943,6 +958,7 @@ ORDER BY
                        {$alias}qa.variant,
                        {$alias}qa.maxmark,
                        {$alias}qa.minfraction,
+                       {$alias}qa.maxfraction,
                        {$alias}qa.flagged,
                        {$alias}qa.questionsummary,
                        {$alias}qa.rightanswer,
@@ -956,15 +972,15 @@ ORDER BY
                        {$alias}qas.userid
 
                   FROM {$qubaids->from_question_attempts($alias . 'qa')}
-                  JOIN {question_attempt_steps} {$alias}qas ON
-                           {$alias}qas.id = {$this->latest_step_for_qa_subquery($alias . 'qa.id')}
+                  JOIN {question_attempt_steps} {$alias}qas ON {$alias}qas.questionattemptid = {$alias}qa.id
+                            AND {$alias}qas.sequencenumber = {$this->latest_step_for_qa_subquery($alias . 'qa.id')}
                  WHERE {$qubaids->where()}
             ) $alias", $qubaids->from_where_params());
     }
 
     protected function latest_step_for_qa_subquery($questionattemptid = 'qa.id') {
         return "(
-                SELECT MAX(id)
+                SELECT MAX(sequencenumber)
                 FROM {question_attempt_steps}
                 WHERE questionattemptid = $questionattemptid
             )";
@@ -1460,6 +1476,14 @@ abstract class qubaid_condition {
      * @return the params needed by a query that uses {@link usage_id_in()}.
      */
     public abstract function usage_id_in_params();
+
+    /**
+     * @return string 40-character hash code that uniquely identifies the combination of properties and class name of this qubaid
+     *                  condition.
+     */
+    public function get_hash_code() {
+        return sha1(serialize($this));
+    }
 }
 
 
