@@ -17,11 +17,7 @@ function afterburner_process_css($css, $theme) {
     return $css;
 }
 
-function afterburner_set_logo($css, $logo, $theme = null) {
-    global $OUTPUT;
-    if ($theme === null) {
-        $theme = $OUTPUT;
-    }
+function afterburner_set_logo($css, $logo, $theme) {
     $tag = '[[setting:logo]]';
     $replacement = $logo;
     if (is_null($replacement)) {
