@@ -51,21 +51,25 @@ class core_user_external extends external_api {
                 'users' => new external_multiple_structure(
                     new external_single_structure(
                         array(
-                            'username'    => new external_value(PARAM_USERNAME, 'Username policy is defined in Moodle security config.'),
-                            'password'    => new external_value(PARAM_RAW, 'Plain text password consisting of any characters'),
-                            'firstname'   => new external_value(PARAM_NOTAGS, 'The first name(s) of the user'),
-                            'lastname'    => new external_value(PARAM_NOTAGS, 'The family name of the user'),
-                            'email'       => new external_value(PARAM_EMAIL, 'A valid and unique email address'),
-                            'auth'        => new external_value(PARAM_PLUGIN, 'Auth plugins include manual, ldap, imap, etc', VALUE_DEFAULT, 'manual', NULL_NOT_ALLOWED),
-                            'idnumber'    => new external_value(PARAM_RAW, 'An arbitrary ID code number perhaps from the institution', VALUE_DEFAULT, ''),
-                            'lang'        => new external_value(PARAM_SAFEDIR, 'Language code such as "en", must exist on server', VALUE_DEFAULT, $CFG->lang, NULL_NOT_ALLOWED),
-                            'theme'       => new external_value(PARAM_PLUGIN, 'Theme name such as "standard", must exist on server', VALUE_OPTIONAL),
-                            'timezone'    => new external_value(PARAM_TIMEZONE, 'Timezone code such as Australia/Perth, or 99 for default', VALUE_OPTIONAL),
-                            'mailformat'  => new external_value(PARAM_INT, 'Mail format code is 0 for plain text, 1 for HTML etc', VALUE_OPTIONAL),
-                            'description' => new external_value(PARAM_TEXT, 'User profile description, no HTML', VALUE_OPTIONAL),
-                            'city'        => new external_value(PARAM_NOTAGS, 'Home city of the user', VALUE_OPTIONAL),
-                            'country'     => new external_value(PARAM_ALPHA, 'Home country code of the user, such as AU or CZ', VALUE_OPTIONAL),
-                            'preferences' => new external_multiple_structure(
+                            'username'            => new external_value(PARAM_USERNAME, 'Username policy is defined in Moodle security config.'),
+                            'password'            => new external_value(PARAM_RAW, 'Plain text password consisting of any characters'),
+                            'firstname'           => new external_value(PARAM_NOTAGS, 'The first name(s) of the user'),
+                            'lastname'            => new external_value(PARAM_NOTAGS, 'The family name of the user'),
+                            'email'               => new external_value(PARAM_EMAIL, 'A valid and unique email address'),
+                            'auth'                => new external_value(PARAM_PLUGIN, 'Auth plugins include manual, ldap, imap, etc', VALUE_DEFAULT, 'manual', NULL_NOT_ALLOWED),
+                            'idnumber'            => new external_value(PARAM_RAW, 'An arbitrary ID code number perhaps from the institution', VALUE_DEFAULT, ''),
+                            'lang'                => new external_value(PARAM_SAFEDIR, 'Language code such as "en", must exist on server', VALUE_DEFAULT, $CFG->lang, NULL_NOT_ALLOWED),
+                            'theme'               => new external_value(PARAM_PLUGIN, 'Theme name such as "standard", must exist on server', VALUE_OPTIONAL),
+                            'timezone'            => new external_value(PARAM_TIMEZONE, 'Timezone code such as Australia/Perth, or 99 for default', VALUE_OPTIONAL),
+                            'mailformat'          => new external_value(PARAM_INT, 'Mail format code is 0 for plain text, 1 for HTML etc', VALUE_OPTIONAL),
+                            'description'         => new external_value(PARAM_TEXT, 'User profile description, no HTML', VALUE_OPTIONAL),
+                            'city'                => new external_value(PARAM_NOTAGS, 'Home city of the user', VALUE_OPTIONAL),
+                            'country'             => new external_value(PARAM_ALPHA, 'Home country code of the user, such as AU or CZ', VALUE_OPTIONAL),
+                            'firstnamephonetic'   => new external_value(PARAM_NOTAGS, 'The first name(s) phonetically of the user', VALUE_OPTIONAL),
+                            'lastnamephonetic'    => new external_value(PARAM_NOTAGS, 'The family name phonetically of the user', VALUE_OPTIONAL),
+                            'middlename'          => new external_value(PARAM_NOTAGS, 'The middle name of the user', VALUE_OPTIONAL),
+                            'alternatename'       => new external_value(PARAM_NOTAGS, 'The alternate name of the user', VALUE_OPTIONAL),
+                            'preferences'         => new external_multiple_structure(
                                 new external_single_structure(
                                     array(
                                         'type'  => new external_value(PARAM_ALPHANUMEXT, 'The name of the preference'),
@@ -108,11 +112,11 @@ class core_user_external extends external_api {
         // If any problems are found then exceptions are thrown with helpful error messages
         $params = self::validate_parameters(self::create_users_parameters(), array('users'=>$users));
 
-        $availableauths  = get_plugin_list('auth');
+        $availableauths  = core_component::get_plugin_list('auth');
         unset($availableauths['mnet']);       // these would need mnethostid too
         unset($availableauths['webservice']); // we do not want new webservice users for now
 
-        $availablethemes = get_plugin_list('theme');
+        $availablethemes = core_component::get_plugin_list('theme');
         $availablelangs  = get_string_manager()->get_list_of_translations();
 
         $transaction = $DB->start_delegated_transaction();
@@ -276,21 +280,25 @@ class core_user_external extends external_api {
                     new external_single_structure(
                         array(
                             'id'    => new external_value(PARAM_INT, 'ID of the user'),
-                            'username'    => new external_value(PARAM_USERNAME, 'Username policy is defined in Moodle security config.', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
-                            'password'    => new external_value(PARAM_RAW, 'Plain text password consisting of any characters', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
-                            'firstname'   => new external_value(PARAM_NOTAGS, 'The first name(s) of the user', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
-                            'lastname'    => new external_value(PARAM_NOTAGS, 'The family name of the user', VALUE_OPTIONAL),
-                            'email'       => new external_value(PARAM_EMAIL, 'A valid and unique email address', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
-                            'auth'        => new external_value(PARAM_PLUGIN, 'Auth plugins include manual, ldap, imap, etc', VALUE_OPTIONAL, '', NULL_NOT_ALLOWED),
-                            'idnumber'    => new external_value(PARAM_RAW, 'An arbitrary ID code number perhaps from the institution', VALUE_OPTIONAL),
-                            'lang'        => new external_value(PARAM_SAFEDIR, 'Language code such as "en", must exist on server', VALUE_OPTIONAL, '', NULL_NOT_ALLOWED),
-                            'theme'       => new external_value(PARAM_PLUGIN, 'Theme name such as "standard", must exist on server', VALUE_OPTIONAL),
-                            'timezone'    => new external_value(PARAM_TIMEZONE, 'Timezone code such as Australia/Perth, or 99 for default', VALUE_OPTIONAL),
-                            'mailformat'  => new external_value(PARAM_INT, 'Mail format code is 0 for plain text, 1 for HTML etc', VALUE_OPTIONAL),
-                            'description' => new external_value(PARAM_TEXT, 'User profile description, no HTML', VALUE_OPTIONAL),
-                            'city'        => new external_value(PARAM_NOTAGS, 'Home city of the user', VALUE_OPTIONAL),
-                            'country'     => new external_value(PARAM_ALPHA, 'Home country code of the user, such as AU or CZ', VALUE_OPTIONAL),
-                            'customfields' => new external_multiple_structure(
+                            'username'            => new external_value(PARAM_USERNAME, 'Username policy is defined in Moodle security config.', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
+                            'password'            => new external_value(PARAM_RAW, 'Plain text password consisting of any characters', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
+                            'firstname'           => new external_value(PARAM_NOTAGS, 'The first name(s) of the user', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
+                            'lastname'            => new external_value(PARAM_NOTAGS, 'The family name of the user', VALUE_OPTIONAL),
+                            'email'               => new external_value(PARAM_EMAIL, 'A valid and unique email address', VALUE_OPTIONAL, '',NULL_NOT_ALLOWED),
+                            'auth'                => new external_value(PARAM_PLUGIN, 'Auth plugins include manual, ldap, imap, etc', VALUE_OPTIONAL, '', NULL_NOT_ALLOWED),
+                            'idnumber'            => new external_value(PARAM_RAW, 'An arbitrary ID code number perhaps from the institution', VALUE_OPTIONAL),
+                            'lang'                => new external_value(PARAM_SAFEDIR, 'Language code such as "en", must exist on server', VALUE_OPTIONAL, '', NULL_NOT_ALLOWED),
+                            'theme'               => new external_value(PARAM_PLUGIN, 'Theme name such as "standard", must exist on server', VALUE_OPTIONAL),
+                            'timezone'            => new external_value(PARAM_TIMEZONE, 'Timezone code such as Australia/Perth, or 99 for default', VALUE_OPTIONAL),
+                            'mailformat'          => new external_value(PARAM_INT, 'Mail format code is 0 for plain text, 1 for HTML etc', VALUE_OPTIONAL),
+                            'description'         => new external_value(PARAM_TEXT, 'User profile description, no HTML', VALUE_OPTIONAL),
+                            'city'                => new external_value(PARAM_NOTAGS, 'Home city of the user', VALUE_OPTIONAL),
+                            'country'             => new external_value(PARAM_ALPHA, 'Home country code of the user, such as AU or CZ', VALUE_OPTIONAL),
+                            'firstnamephonetic'   => new external_value(PARAM_NOTAGS, 'The first name(s) phonetically of the user', VALUE_OPTIONAL),
+                            'lastnamephonetic'    => new external_value(PARAM_NOTAGS, 'The family name phonetically of the user', VALUE_OPTIONAL),
+                            'middlename'          => new external_value(PARAM_NOTAGS, 'The middle name of the user', VALUE_OPTIONAL),
+                            'alternatename'       => new external_value(PARAM_NOTAGS, 'The alternate name of the user', VALUE_OPTIONAL),
+                            'customfields'        => new external_multiple_structure(
                                 new external_single_structure(
                                     array(
                                         'type'  => new external_value(PARAM_ALPHANUMEXT, 'The name of the custom field'),
@@ -670,20 +678,22 @@ class core_user_external extends external_api {
         $params = self::validate_parameters(self::get_users_by_id_parameters(),
                 array('userids'=>$userids));
 
-        list($uselect, $ujoin) = context_instance_preload_sql('u.id', CONTEXT_USER, 'ctx');
-        list($sqluserids, $params) = $DB->get_in_or_equal($userids);
+        list($sqluserids, $params) = $DB->get_in_or_equal($userids, SQL_PARAMS_NAMED);
+        $uselect = ', ' . context_helper::get_preload_record_columns_sql('ctx');
+        $ujoin = "LEFT JOIN {context} ctx ON (ctx.instanceid = u.id AND ctx.contextlevel = :contextlevel)";
+        $params['contextlevel'] = CONTEXT_USER;
         $usersql = "SELECT u.* $uselect
                       FROM {user} u $ujoin
                      WHERE u.id $sqluserids";
         $users = $DB->get_recordset_sql($usersql, $params);
 
         $result = array();
-        $hasuserupdatecap = has_capability('moodle/user:update', get_system_context());
+        $hasuserupdatecap = has_capability('moodle/user:update', context_system::instance());
         foreach ($users as $user) {
             if (!empty($user->deleted)) {
                 continue;
             }
-            context_instance_preload($user);
+            context_helper::preload_from_record($user);
             $usercontext = context_user::instance($user->id, IGNORE_MISSING);
             self::validate_context($usercontext);
             $currentuser = ($user->id == $USER->id);
@@ -770,22 +780,26 @@ class core_user_external extends external_api {
 
         // cache all courses
         $courses = array();
-        list($cselect, $cjoin) = context_instance_preload_sql('c.id', CONTEXT_COURSE, 'ctx');
-        list($sqlcourseids, $params) = $DB->get_in_or_equal(array_unique($courseids));
+        list($sqlcourseids, $params) = $DB->get_in_or_equal(array_unique($courseids), SQL_PARAMS_NAMED);
+        $cselect = ', ' . context_helper::get_preload_record_columns_sql('ctx');
+        $cjoin = "LEFT JOIN {context} ctx ON (ctx.instanceid = c.id AND ctx.contextlevel = :contextlevel)";
+        $params['contextlevel'] = CONTEXT_COURSE;
         $coursesql = "SELECT c.* $cselect
                         FROM {course} c $cjoin
                        WHERE c.id $sqlcourseids";
         $rs = $DB->get_recordset_sql($coursesql, $params);
         foreach ($rs as $course) {
             // adding course contexts to cache
-            context_instance_preload($course);
+            context_helper::preload_from_record($course);
             // cache courses
             $courses[$course->id] = $course;
         }
         $rs->close();
 
-        list($uselect, $ujoin) = context_instance_preload_sql('u.id', CONTEXT_USER, 'ctx');
-        list($sqluserids, $params) = $DB->get_in_or_equal($userids);
+        list($sqluserids, $params) = $DB->get_in_or_equal($userids, SQL_PARAMS_NAMED);
+        $uselect = ', ' . context_helper::get_preload_record_columns_sql('ctx');
+        $ujoin = "LEFT JOIN {context} ctx ON (ctx.instanceid = u.id AND ctx.contextlevel = :contextlevel)";
+        $params['contextlevel'] = CONTEXT_USER;
         $usersql = "SELECT u.* $uselect
                       FROM {user} u $ujoin
                      WHERE u.id $sqluserids";
@@ -795,7 +809,7 @@ class core_user_external extends external_api {
             if (!empty($user->deleted)) {
                 continue;
             }
-            context_instance_preload($user);
+            context_helper::preload_from_record($user);
             $course = $courses[$courseids[$user->id]];
             $context = context_course::instance($courseids[$user->id], IGNORE_MISSING);
             self::validate_context($context);
@@ -912,6 +926,168 @@ class core_user_external extends external_api {
         return new external_single_structure($userfields);
     }
 
+    /**
+     * Returns description of method parameters
+     *
+     * @return external_function_parameters
+     * @since Moodle 2.6
+     */
+    public static function add_user_private_files_parameters() {
+        return new external_function_parameters(
+            array(
+                'draftid' => new external_value(PARAM_INT, 'draft area id')
+            )
+        );
+    }
+
+    /**
+     * Copy files from a draft area to users private files area.
+     *
+     * @param int $draftid Id of a draft area containing files.
+     * @return array An array of warnings
+     * @since Moodle 2.6
+     */
+    public static function add_user_private_files($draftid) {
+        global $CFG, $USER, $DB;
+
+        require_once($CFG->dirroot . "/user/lib.php");
+        $params = self::validate_parameters(self::add_user_private_files_parameters(), array('draftid'=>$draftid));
+
+        if (isguestuser()) {
+            throw new invalid_parameter_exception('Guest users cannot upload files');
+        }
+
+        $context = context_user::instance($USER->id);
+        require_capability('moodle/user:manageownfiles', $context);
+
+        $maxbytes = $CFG->userquota;
+        $maxareabytes = $CFG->userquota;
+        if (has_capability('moodle/user:ignoreuserquota', $context)) {
+            $maxbytes = USER_CAN_IGNORE_FILE_SIZE_LIMITS;
+            $maxareabytes = FILE_AREA_MAX_BYTES_UNLIMITED;
+        }
+
+        $options = array('subdirs' => 1,
+                         'maxbytes' => $maxbytes,
+                         'maxfiles' => -1,
+                         'accepted_types' => '*',
+                         'areamaxbytes' => $maxareabytes);
+
+        file_save_draft_area_files($draftid, $context->id, 'user', 'private', 0, $options);
+
+        return null;
+    }
+
+    /**
+     * Returns description of method result value
+     *
+     * @return external_description
+     * @since Moodle 2.2
+     */
+    public static function add_user_private_files_returns() {
+        return null;
+    }
+
+    /**
+     * Returns description of method parameters.
+     *
+     * @return external_function_parameters
+     * @since Moodle 2.6
+     */
+    public static function add_user_device_parameters() {
+        return new external_function_parameters(
+            array(
+                'appid'     => new external_value(PARAM_NOTAGS, 'the app id, usually something like com.moodle.moodlemobile'),
+                'name'      => new external_value(PARAM_NOTAGS, 'the device name, \'occam\' or \'iPhone\' etc.'),
+                'model'     => new external_value(PARAM_NOTAGS, 'the device model \'Nexus4\' or \'iPad1,1\' etc.'),
+                'platform'  => new external_value(PARAM_NOTAGS, 'the device platform \'iOS\' or \'Android\' etc.'),
+                'version'   => new external_value(PARAM_NOTAGS, 'the device version \'6.1.2\' or \'4.2.2\' etc.'),
+                'pushid'    => new external_value(PARAM_RAW, 'the device PUSH token/key/identifier/registration id'),
+                'uuid'      => new external_value(PARAM_RAW, 'the device UUID')
+            )
+        );
+    }
+
+    /**
+     * Add a user device in Moodle database (for PUSH notifications usually).
+     *
+     * @param string $appid The app id, usually something like com.moodle.moodlemobile.
+     * @param string $name The device name, occam or iPhone etc.
+     * @param string $model The device model Nexus4 or iPad1.1 etc.
+     * @param string $platform The device platform iOs or Android etc.
+     * @param string $version The device version 6.1.2 or 4.2.2 etc.
+     * @param string $pushid The device PUSH token/key/identifier/registration id.
+     * @param string $uuid The device UUID.
+     * @return array List of possible warnings.
+     * @since Moodle 2.6
+     */
+    public static function add_user_device($appid, $name, $model, $platform, $version, $pushid, $uuid) {
+        global $CFG, $USER, $DB;
+        require_once($CFG->dirroot . "/user/lib.php");
+
+        $params = self::validate_parameters(self::add_user_device_parameters(),
+                array('appid' => $appid,
+                      'name' => $name,
+                      'model' => $model,
+                      'platform' => $platform,
+                      'version' => $version,
+                      'pushid' => $pushid,
+                      'uuid' => $uuid
+                      ));
+
+        $warnings = array();
+
+        // Prevent duplicate keys for users.
+        if ($DB->get_record('user_devices', array('pushid' => $params['pushid'], 'userid' => $USER->id))) {
+            $warnings['warning'][] = array(
+                'item' => $params['pushid'],
+                'warningcode' => 'existingkeyforthisuser',
+                'message' => 'This key is already stored for this user'
+            );
+            return $warnings;
+        }
+
+        // The same key can't exists for the same platform.
+        if ($DB->get_record('user_devices', array('pushid' => $params['pushid'], 'platform' => $params['platform']))) {
+            $warnings['warning'][] = array(
+                'item' => $params['pushid'],
+                'warningcode' => 'existingkeyforplatform',
+                'message' => 'This key is already stored for other device using the same platform'
+            );
+            return $warnings;
+        }
+
+        $userdevice = new stdclass;
+        $userdevice->userid     = $USER->id;
+        $userdevice->appid      = $params['appid'];
+        $userdevice->name       = $params['name'];
+        $userdevice->model      = $params['model'];
+        $userdevice->platform   = $params['platform'];
+        $userdevice->version    = $params['version'];
+        $userdevice->pushid     = $params['pushid'];
+        $userdevice->uuid       = $params['uuid'];
+        $userdevice->timecreated  = time();
+        $userdevice->timemodified = $userdevice->timecreated;
+
+        if (!$DB->insert_record('user_devices', $userdevice)) {
+            throw new moodle_exception("There was a problem saving in the database the device with key: " . $params['pushid']);
+        }
+
+        return $warnings;
+    }
+
+    /**
+     * Returns description of method result value.
+     *
+     * @return external_multiple_structure
+     * @since Moodle 2.6
+     */
+    public static function add_user_device_returns() {
+        return new external_multiple_structure(
+           new external_warnings()
+        );
+    }
+
 }
 
  /**
